<?php

namespace Drupal\prelinker\Render;

use Drupal\Core\Render\AttachmentsInterface;
use Drupal\Core\Render\AttachmentsResponseProcessorInterface;

/**
 * Adds Server Push link headers.
 */
class HeaderProcessor implements AttachmentsResponseProcessorInterface
{
    /**
     * The decorated HTML response attachments processor service.
     *
     * @var \Drupal\Core\Render\AttachmentsResponseProcessorInterface
     */
    protected $attachmentsProcessor;

    /**
     * Constructs a HtmlResponseAttachmentsProcessor object.
     *
     * @param \Drupal\Core\Render\AttachmentsResponseProcessorInterface $attachmentsProcessor
     */
    public function __construct(AttachmentsResponseProcessorInterface $attachmentsProcessor)
    {
        $this->attachmentsProcessor = $attachmentsProcessor;
    }

    /**
     * {@inheritdoc}
     */
    public function processAttachments(AttachmentsInterface $response)
    {
        $response = $this->attachmentsProcessor->processAttachments($response);

        // don't add to admin pages
        if (\Drupal::service('router.admin_context')->isAdminRoute()) {
            return $response;
        }

        $config = \Drupal::configFactory()->getEditable('prelinker.settings');

        $links = [];

        // preconnects
        if ($config->get('preconnect_push')) {
            $storage = \Drupal::entityTypeManager()->getStorage('preconnect');
            $query = $storage->getQuery()->sort('weight', 'ASC')->execute();

            $preconnects = $storage->loadMultiple($query);

            foreach ($preconnects as $preconnect) {
                if (\Drupal::service('prelinker.prelinker')->check_page($preconnect->get('pages'))) {
                    $links[] = '<'.$preconnect->get('domain').'>; rel="preconnect"';
                }
            }
        }

        // preloads
        if ($config->get('preload_push')) {
            $storage = \Drupal::entityTypeManager()->getStorage('preload');
            $query = $storage->getQuery()->sort('weight', 'ASC')->execute();

            $preloads = $storage->loadMultiple($query);

            foreach ($preloads as $preload) {
                if (\Drupal::service('prelinker.prelinker')->check_page($preload->get('pages'))) {
                    $links[] = '<'.$preload->get('file').'>; rel="preload"; as="'.$preload->get('as')
                        .($preload->get('as') == 'font'? '; crossorigin=crossorigin': '').'"';
                }
            }
        }
        // new links to add to the headers?
        if ($links) {
            $current = $response->headers->get('Link', null, false);
            $merged = array_unique(array_merge($current, $links));
            $response->headers->set('Link', $merged, true);
        }

        return $response;
    }
}
